﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Globalization;

using Com.Piwigo.Lib.DTO;
using Com.Piwigo.Lib.Proxy;
using Com.Piwigo.Lib.Proxy.Response;
using Com.Piwigo.Lib.DTO.Helper;
using Com.Piwigo.Lib.IService;
using System.IO;

namespace Com.Piwigo.Lib.Service
{

    internal sealed class PwgImagesService : IPwgImagesService
    {
        public PwgImageRate RateImage(Int32 imageId, Int32 imageRate)
        {
            PwgImageRate returnValue = new PwgImageRate();

            try
            {
                PwgImageRateProxyResponse response = PwgImagesProxy.pwg_images_rate(imageId, imageRate);

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("RateImage, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("RateImage : a error occurs during server process.");
                    }
                }
                else
                {
                    returnValue = ConvertProxyResponseToDTO(response);
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("RateImage : a error is raised by proxy.", ex);
            }
            return returnValue;
        }

        public PwgImageAdded addImageByMultiPartForm(FileInfo fileImage, Int32? imageId, Int32? categoryId, String imageName, String imageAuthor, String imageComment, Int32? imageLevel, List<PwgTag> imageTags)
        {
            PwgImageAdded returnValue = new PwgImageAdded();

            try
            {
                PwgAddSimpleImageProxyResponse response = PwgImagesProxy.pwg_images_addSimple(fileImage, imageId, categoryId, imageName, imageAuthor, imageComment, imageLevel, imageTags);

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("addImageByMultiPartForm, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("addImageByMultiPartForm : a error occurs during server process.");
                    }
                }
                else
                {
                    returnValue = ConvertProxyResponseToDTO(response);
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("addImageByMultiPartForm : a error is raised by proxy.", ex);
            }
            return returnValue;
        }

        public Boolean addImage(FileInfo highResFile, FileInfo lowResFile, FileInfo thumbFile, String imageName, String imageAuthor, DateTime creationDate,
            String authorComment, List<String> lstCategories, List<String> lstTags, PwgConfidentLevelEnum? confidentLevel)
        {
            Boolean returnValue = false;

            try
            {
                String fileGuid =  uploadImageByChunk(lowResFile, PwgFileTypeEnum.LowRes);
                String thunbGuid = uploadImageByChunk(thumbFile, PwgFileTypeEnum.Thumb);
                String HighresGuid = uploadImageByChunk(highResFile, PwgFileTypeEnum.HighDef);
                String imageGuid = (highResFile != null ? HighresGuid : fileGuid);  //Guid.NewGuid().ToString();
                String imageFilename = (highResFile != null ? highResFile.Name : lowResFile.Name);

                PwgBaseProxyReponse response = PwgImagesProxy.pwg_images_add( fileGuid, thunbGuid, HighresGuid, 
                                                    imageGuid,  imageFilename, imageName, imageAuthor, creationDate,  
                                                    authorComment, lstCategories, lstTags, confidentLevel);

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("addImage, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("addImage : a error occurs during server process.");
                    }
                }
                else
                {
                    returnValue = true;
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("addImage : a error is raised by proxy.", ex);
            }
            return returnValue;
        }

        public String uploadImageByChunk(FileInfo imageFile, PwgFileTypeEnum fileType)
        {
            byte[] buffer = new byte[5000];
            string strGuid = String.Empty; ;
            int read;
            int chunkNumber = 1;

            try
            {
                if (imageFile != null)
                {
                    FileStream fs = File.OpenRead(imageFile.FullName);
                    strGuid = Com.Piwigo.Lib.Proxy.Helper.PwgBase64Helper.GetphpMd5Sum(fs);
                    
                    fs.Seek(0, SeekOrigin.Begin);
                    while ((read = fs.Read(buffer, 0, buffer.Length)) > 0)
                    {
                        String strB64Data = Com.Piwigo.Lib.Proxy.Helper.PwgBase64Helper.base64Encode(buffer);
                        PwgBaseProxyReponse response = PwgImagesProxy.pwg_images_addChunk(strB64Data, strGuid, fileType, chunkNumber);

                        chunkNumber += 1;

                        if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                        {
                            if (response.Erreur != null)
                            {
                                throw new PwgServiceException("uploadImageByChunk, the server has return the error.",
                                    response.Erreur.Code,
                                    response.Erreur.Message);
                            }
                            else
                            {
                                throw new PwgServiceException("uploadImageByChunk : a error occurs during server process.");
                            }
                        }
                    }
                }
                else
                {
                    strGuid = String.Empty;
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("uploadImageByChunk : a error is raised by proxy.", ex);
            }
            return strGuid;
        }

        /// <summary>
        /// Delete a image
        /// </summary>
        /// <param name="imageId"></param>
        /// <param name="SecurityToken"></param>
        /// <returns></returns>
        public Boolean DeleteImage(Int32 imageId, String SecurityToken)
        {
            Boolean returnValue = false;

            try
            {

                PwgBaseProxyReponse response = PwgImagesProxy.pwg_images_delete(imageId, SecurityToken);

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("DeleteImage, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("DeleteImage : a error occurs during server process.");
                    }
                }
                else
                {
                    returnValue = true;
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("DeleteImage : a error is raised by proxy.", ex);
            }
            return returnValue;
        }

        public PwgImageInfo getImageInfo(Int32 imageId, Int32? commentPage, Int32? commentsPerPage)
        {
            PwgImageInfo returnValue = new PwgImageInfo();

            try
            {
                PwgImageInfoProxyResponse response = PwgImagesProxy.pwg_images_getInfos(imageId, commentPage, commentsPerPage);

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("RateImage, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("RateImage : a error occurs during server process.");
                    }
                }
                else
                {
                    returnValue = ConvertProxyResponseToDTO(response.ImageInfo);
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("RateImage : a error is raised by proxy.", ex);
            }
            return returnValue;
        }        

        /// <summary>
        /// private: convert response to dto object
        /// </summary>
        /// <param name="response"></param>
        /// <param name="session"></param>
        internal static PwgImageAdded ConvertProxyResponseToDTO(PwgAddSimpleImageProxyResponse response)
        {
            PwgImageAdded returnValue = new PwgImageAdded();

            returnValue.Id = response.Id;

            if (String.IsNullOrEmpty(response.UrlImage))
            {
                returnValue.UrlElement = null;
            }
            else
            {
                returnValue.UrlElement = (new UriBuilder(response.UrlImage)).Uri;
            }
            
            return returnValue;
        }


        /// <summary>
        /// private: convert response to dto object
        /// </summary>
        /// <param name="response"></param>
        /// <param name="session"></param>
        internal static PwgImageRate ConvertProxyResponseToDTO(PwgImageRateProxyResponse response)
        {
            PwgImageRate returnValue = new PwgImageRate();
            Double aDouble = 0;
            NumberStyles style = NumberStyles.AllowDecimalPoint;
            CultureInfo en_us = CultureInfo.CreateSpecificCulture("en-US");

            returnValue.Count = response.Count;

            returnValue.Stdev = 0.0 ;
            if (Double.TryParse(response.Stdev, style, en_us, out aDouble))
            {
                returnValue.Stdev = aDouble;
            }

            returnValue.Average = 0.0;
            if (Double.TryParse(response.Average, style, en_us, out aDouble))
            {
                returnValue.Average = aDouble;
            }


            return returnValue;
        }

        /// <summary>
        /// private: convert response to dto object
        /// </summary>
        /// <param name="response"></param>
        /// <param name="session"></param>
        internal static void ConvertProxyResponseToDTO(PwgImagesProxyResponse response,
                                                      List<PwgImage> lstImage, 
                                                      ref Int32 Page, ref Int32 PerPagee, ref Int32 Count)
        {
                ConvertProxyResponseToDTO(response.ImageList, lstImage, ref Page, ref PerPagee, ref Count);
        }

        /// <summary>
        /// private: convert response to dto object
        /// </summary>
        /// <param name="response"></param>
        /// <param name="session"></param>
        internal static void ConvertProxyResponseToDTO(PwgImageListProxyResponse response,
                                                      List<PwgImage> lstImage, 
                                                      ref Int32 Page, ref Int32 PerPagee, ref Int32 Count)
        {
            Page = response.Page;
            PerPagee = response.PerPage;
            Count = response.Count;

            if (response.Images != null)
            {
                foreach (PwgImageProxyResponse respImg in response.Images)
                {
                    lstImage.Add(ConvertProxyResponseToDTO(respImg));
                }
            }
        }

        /// <summary>
        /// private: convert response to dto object
        /// </summary>
        /// <param name="response"></param>
        /// <param name="session"></param>
        internal static PwgImageInfo ConvertProxyResponseToDTO(PwgImageInfoImageProxyResponse response)
        {
            PwgImageInfo returnValue = null;

            try
            {
                PwgImage pwgImg = ConvertProxyResponseToDTO(response as PwgImageProxyResponse as PwgImageProxyResponse);
                returnValue = new PwgImageInfo(){   Id           = pwgImg.Id,
                                                    Width        = pwgImg.Width,
                                                    Height       = pwgImg.Height,
                                                    File         = pwgImg.File ,
                                                    UrlThunb     = pwgImg.UrlThunb,
                                                    UrlElement   = pwgImg.UrlElement,
                                                    UrlHighDef   = pwgImg.UrlHighDef,
                                                    Counter      = pwgImg.Counter,
                                                    CreatingDate = pwgImg.CreatingDate,
                                                    AvailableDate= pwgImg.AvailableDate,
                                                    Tags         = pwgImg.Tags,
                                                    Categories   = pwgImg.Categories};

               returnValue.ExtThumbail       = response.ExtThumbail       ;
               returnValue.Author            = response.Author            ;
               returnValue.ImageFileSize     = response.ImageFileSize     ;
               returnValue.hasHighDef        = response.hasHighDef        ;
               returnValue.HighDefFileSize   = response.HighDefFileSize   ;
               returnValue.ConfidentialLevel = response.ConfidentialLevel ;
               returnValue.md5Sum            = response.md5Sum            ;
               returnValue.UserIdAddedBy     = response.UserIdAddedBy     ;
               returnValue.Name              = response.Name              ;
               
                if (response.DateUpdateMetaData != null)
               {
                   DateTime tryDate;
                   if (DateTime.TryParse(response.DateUpdateMetaData, out tryDate))
                   {
                       returnValue.DateUpdateMetaData = tryDate;
                   }
               }

               if (response.ImageRate != null)
               {
                    returnValue.ImageRate = ConvertProxyResponseToDTO(response.ImageRate);
               }
               if (response.ImageComments != null)
               {
                    returnValue.ImageComments = ConvertProxyResponseToDTO(response.ImageComments);
               }
               if (response.PostCommentSecurityData != null)
               {
                   returnValue.PostCommenSecurityData = ConvertProxyResponseToDTO(response.PostCommentSecurityData);
               }
            }
            catch (Exception ex)
            {
                throw new PwgServiceException("ConvertProxyResponseToDTO : a error is raised when converting PwgTagProxyResponse.", ex);
            }

            return returnValue;
        }

        /// <summary>
        /// private: convert response to dto object
        /// </summary>
        /// <param name="response"></param>
        /// <param name="session"></param>
        internal static PwgPostCommentSecurityInfo ConvertProxyResponseToDTO(PwgPostCommentSecurityInfo response)
        {
            PwgPostCommentSecurityInfo returnValue = new PwgPostCommentSecurityInfo();

            returnValue.Author = response.Author;
            returnValue.AllowPostKey = response.AllowPostKey;

            return returnValue;
        }

        /// <summary>
        /// private: convert response to dto object
        /// </summary>
        /// <param name="response"></param>
        /// <param name="session"></param>
        internal static PwgCommentPage ConvertProxyResponseToDTO(PwgCommentPageProxyResponse response)
        {
            PwgCommentPage returnValue = new PwgCommentPage();
    
            returnValue.Page = response.Page;
            returnValue.PerPage = response.PerPage;
            returnValue.CountOf = response.CountOf;
            returnValue.TotalCommentNumber = response.TotalCommentNumber;

            //PwgCommentPage inherit List<PwgComment>
            if (response.Comments != null)
            {
                foreach (PwgCommentProxyResponse respComment in response.Comments)
                {
                    PwgComment pwgCom = ConvertProxyResponseToDTO(respComment);
                    if (pwgCom != null)
                    {
                        returnValue.Add(pwgCom);
                    }
                }
            }

            return returnValue;
        }

        /// <summary>
        /// private: convert response to dto object
        /// </summary>
        /// <param name="response"></param>
        /// <param name="session"></param>
        internal static PwgComment ConvertProxyResponseToDTO(PwgCommentProxyResponse response)
        {
            PwgComment returnValue = new PwgComment();

            returnValue.Id          = response.Id;          
            returnValue.CommentDate = response.CommentDate;
            returnValue.Author      = response.Author;
            returnValue.Content     = response.Content;     

            return returnValue;
        }

        /// <summary>
        /// private: convert response to dto object
        /// </summary>
        /// <param name="response"></param>
        /// <param name="session"></param>
        internal static PwgImage ConvertProxyResponseToDTO(PwgImageProxyResponse response)
        {
            PwgImage returnValue = new PwgImage();

            try
            {
                returnValue.Counter = response.Counter;
                returnValue.Id = response.Id;
                returnValue.File = response.File;
                returnValue.Height = response.Height;
                returnValue.Width = response.Width;

                if (response.AvailableDate != null)
                {
                    DateTime tryDate;
                    if (DateTime.TryParse(response.AvailableDate, out tryDate))
                    {
                        returnValue.AvailableDate = tryDate;
                    }
                }

                if (response.CreatingDate != null)
                {
                    DateTime tryDate;
                    if (DateTime.TryParse(response.CreatingDate, out tryDate))
                    {
                        returnValue.CreatingDate = tryDate;
                    }
                }

                if (String.IsNullOrEmpty(response.UrlElement))
                {
                    returnValue.UrlElement = null;
                }
                else
                {
                    returnValue.UrlElement = (new UriBuilder(response.UrlElement)).Uri;
                }

                if (String.IsNullOrEmpty(response.UrlThunb))
                {
                    returnValue.UrlThunb = null;
                }
                else
                {
                    returnValue.UrlThunb = (new UriBuilder(response.UrlThunb)).Uri;
                }

                if (String.IsNullOrEmpty(response.UrlHighDef))
                {
                    returnValue.UrlHighDef = null;
                }
                else
                {
                    returnValue.UrlHighDef = (new UriBuilder(response.UrlHighDef)).Uri;
                }

                returnValue.Tags = new List<PwgTag>();
                if (response.Tags != null)
                {
                    foreach (PwgTagProxyResponse tagResp in response.Tags)
                    {
                        PwgTag aTag = PwgTagsService.ConvertProxyResponseToDTO(tagResp);
                        returnValue.Tags.Add(aTag);
                    }
                }

                returnValue.Categories = new List<PwgCategory>();
                if (response.Categories != null)
                {
                    foreach (PwgCategoryProxyResponse catResp in response.Categories)
                    {
                        PwgCategory aCat = PwgCategoriesService.ConvertProxyResponseToDTO(catResp);
                        returnValue.Categories.Add(aCat);
                    }
                }

            }
            catch (Exception ex)
            {
                throw new PwgServiceException("ConvertProxyResponseToDTO : a error is raised when converting PwgTagProxyResponse.", ex);
            }

            return returnValue;
        }
    }
}
